"use client";
import { marginX } from "@/utils/constants";
import {
  Flex,
  FormControl,
  FormLabel,
  Heading,
  Input,
  SimpleGrid,
  Stack,
  Text,
  Textarea,
  Box,
  Button,
  useToast,
  FormErrorMessage,
  FormErrorIcon,
} from "@chakra-ui/react";
import Image from "next/image";
import { z } from "zod";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { useState } from "react";
import { handleFormSubmissionAction } from "@/app/actions";
import { CMS_ASSETS_URL } from "@/config";

const schema = z.object({
  name: z
    .string()
    .min(3, { message: "Der Name muss mindestens 3 Zeichen lang sein" }),
  email: z
    .string()
    .email({ message: "Bitte geben Sie eine gültige Email-Adresse ein" }),
  thema: z
    .string()
    .min(3, { message: "Der Betreff muss mindestens 3 Zeichen lang sein" }),
  telefon: z.string().min(10, {
    message: "Die Telefonnummer muss mindestens 10 Zeichen lang sein",
  }),
  nachricht: z
    .string()
    .min(10, { message: "Die Nachricht muss mindestens 10 Zeichen lang sein" }),
});

export type FormData = z.infer<typeof schema>;

interface Props {
  title: string;
  imageId: string;
  description: string;
}

const ContactForm = ({ title, imageId, description }: Props) => {
  const {
    register,
    handleSubmit,
    reset,
    formState: { errors, isValid },
  } = useForm<FormData>({ resolver: zodResolver(schema), mode: "onChange" });
  const [isLoading, setIsLoading] = useState(false);
  const toast = useToast();

  const onSubmit = async (data: FormData) => {
    setIsLoading(true);

    const res = await handleFormSubmissionAction(data);
    setIsLoading(false);
    if (res.status === 200) {
      reset();
      toast({
        title: res.body.message,
        status: "success",
        duration: 5000,
        isClosable: true,
        position: "top-right",
      });
    } else {
      toast({
        title: res.body.message,
        status: "error",
        duration: 5000,
        isClosable: true,
        position: "top-right",
      });
    }
  };

  return (
    <Box bg="brand.lightMain">
      <Stack py="4rem" marginX={marginX} gap={8}>
        <Stack alignSelf="center">
          <Heading textAlign="center" fontSize={{ base: "3xl", md: "5xl" }}>
            {title}
          </Heading>
          <Text textAlign="center" fontSize="lg">
            {description}
          </Text>
        </Stack>

        <SimpleGrid columns={{ base: 1, md: 2 }} gap={12} alignItems="center">
          <Box overflow="hidden" borderRadius="lg">
            <Image
              height={400}
              width={400}
              style={{
                objectFit: "cover",
                width: "100%",
                height: "100%",
              }}
              src={
                `${CMS_ASSETS_URL}/${imageId}` ||
                "/services/service-item-placeholder.jpg"
              }
              alt="Contact us image"
            />
          </Box>
          <Stack as="form" gap={7} onSubmit={handleSubmit(onSubmit)}>
            <Flex gap={8} flexDir={{ base: "column", sm: "row" }}>
              <FormControl isRequired isInvalid={!!errors.name}>
                <FormLabel>Name</FormLabel>
                <Input
                  {...register("name")}
                  borderColor="brand.lightBlue"
                  bg="brand.white"
                  placeholder="ZB: Sarah Taylor"
                />
                {errors.name && (
                  <FormErrorMessage>
                    <FormErrorIcon />
                    {errors.name.message}
                  </FormErrorMessage>
                )}
              </FormControl>
              <FormControl isRequired isInvalid={!!errors.email}>
                <FormLabel>Email</FormLabel>
                <Input
                  borderColor="brand.lightBlue"
                  bg="brand.white"
                  {...register("email")}
                  placeholder="hello@sarah.com"
                />
              </FormControl>
            </Flex>
            <Flex gap={8} flexDir={{ base: "column", sm: "row" }}>
              <FormControl isRequired isInvalid={!!errors.thema}>
                <FormLabel>Thema</FormLabel>
                <Input
                  {...register("thema")}
                  bg="brand.white"
                  borderColor="brand.lightBlue"
                  placeholder="Geben Sie Ihren Betreff ein"
                />
                {errors.thema && (
                  <FormErrorMessage>
                    <FormErrorIcon />
                    {errors.thema.message}
                  </FormErrorMessage>
                )}
              </FormControl>
              <FormControl isRequired isInvalid={!!errors.telefon}>
                <FormLabel>Telefon</FormLabel>
                <Input
                  {...register("telefon")}
                  bg="brand.white"
                  borderColor="brand.lightBlue"
                  placeholder="Gib deine Telefonnummer ein"
                />
                {errors.telefon && (
                  <FormErrorMessage>
                    <FormErrorIcon />
                    {errors.telefon.message}
                  </FormErrorMessage>
                )}
              </FormControl>
            </Flex>
            <FormControl isRequired isInvalid={!!errors.nachricht}>
              <FormLabel>Nachricht</FormLabel>
              <Textarea
                {...register("nachricht")}
                borderColor="brand.lightBlue"
                bg="brand.white"
                placeholder="Geben Sie hier Ihre Nachricht ein"
              />
              {errors.nachricht && (
                <FormErrorMessage>
                  <FormErrorIcon />
                  {errors.nachricht.message}
                </FormErrorMessage>
              )}
            </FormControl>
            <Button
              fontSize="2xl"
              isLoading={isLoading}
              isDisabled={!isValid || isLoading}
              type="submit"
              bg="brand.main"
              w="fit-content"
              py="1.6rem"
              px="2rem"
              borderRadius="none"
              transform="matrix(1, 0, -0.26, 0.97, 0, 0)"
            >
              Kontact
            </Button>
          </Stack>
        </SimpleGrid>
      </Stack>
    </Box>
  );
};

export default ContactForm;
