"use client";
import { MoveFormData, moveSchema } from "@/schemas";
import useMoveRequestStore, { Field, FieldItem } from "@/store";
import { SendIcon } from "@/utils/icons";
import {
  Accordion,
  AccordionButton,
  AccordionIcon,
  AccordionItem,
  AccordionPanel,
  Alert,
  AlertIcon,
  Box,
  Button,
  Flex,
  FormControl,
  FormErrorIcon,
  FormErrorMessage,
  FormLabel,
  Heading,
  Input,
  Radio,
  RadioGroup,
  SimpleGrid,
  Stack,
  Table,
  TableContainer,
  Tbody,
  Td,
  Text,
  Tfoot,
  Th,
  Thead,
  Tr,
  useToast,
} from "@chakra-ui/react";
import { zodResolver } from "@hookform/resolvers/zod";
import { useState } from "react";
import { useForm } from "react-hook-form";
import { CustomDivider, CustomTh } from "../common";
import FormHeading from "./form-heading";
import SonstigenTextArea from "./sonstigen";
import { handleMoveRequestAction } from "@/actions/move-action";

const MoveForm = () => {
  const [isLoading, setIsLoading] = useState(false);
  const toast = useToast();
  const {
    register,
    handleSubmit,
    reset,
    formState: { errors, isValid },
  } = useForm<MoveFormData>({
    resolver: zodResolver(moveSchema),
    mode: "onChange",
  });
  const {
    wohnzimmer,
    arbeitszimmer,
    kinderzimmer,
    diele,
    esszimmer,
    schlafzimmer,
    kuche,
    keller,
    garage,
    total,
    resetStore,
    modify,
  } = useMoveRequestStore();

  const tableData: {
    data: Field[];
    heading: string;
    type: keyof FieldItem;
  }[] = [
    {
      data: wohnzimmer.fields,
      heading: "Wohn- / Esszimmer",
      type: "wohnzimmer",
    },
    {
      data: arbeitszimmer.fields,
      heading: "Arbeitszimmer",
      type: "arbeitszimmer",
    },
    {
      data: kinderzimmer.fields,
      heading: "Kinderzimmer",
      type: "kinderzimmer",
    },
    {
      data: diele.fields,
      heading: "Diele / Bad ",
      type: "diele",
    },
    {
      data: esszimmer.fields,
      heading: "Eßzimmer",
      type: "esszimmer",
    },
    {
      data: schlafzimmer.fields,
      heading: "Schlafzimmer",
      type: "schlafzimmer",
    },
    {
      data: kuche.fields,
      heading: "Küche",
      type: "kuche",
    },
    {
      data: keller.fields,
      heading: "Keller / Speicher",
      type: "keller",
    },
    {
      data: garage.fields,

      heading: "Garage / Garten / Balkon",
      type: "garage",
    },
  ];

  async function onSubmit(data: MoveFormData) {
    setIsLoading(true);

    const res = await handleMoveRequestAction(
      data,
      wohnzimmer,
      arbeitszimmer,
      kinderzimmer,
      diele,
      esszimmer,
      schlafzimmer,
      kuche,
      keller,
      garage
    );

    setIsLoading(false);

    if (res.status === 200) {
      reset();
      resetStore();
      toast({
        title: res.body.message,
        status: "success",
        duration: 5000,
        isClosable: true,
        position: "top",
      });
    } else {
      toast({
        title: res.body.message,
        status: "error",
        duration: 5000,
        isClosable: true,
        position: "top-right",
      });
    }
  }

  function getTotal(type: keyof FieldItem) {
    return tableData
      .find((data) => data.type === type)
      ?.data.reduce((acc, curr) => acc + curr.gesRE, 0);
  }

  function getSectionDataTotals(type: keyof FieldItem) {
    return tableData
      .find((data) => data.type === type)
      ?.data.reduce((acc, curr) => acc + curr.number, 0);
  }

  function getOverallTotal() {
    return tableData.reduce((acc, curr) => {
      return (
        acc +
        curr.data.reduce((acc, curr) => {
          return acc + curr.gesRE;
        }, 0)
      );
    }, 0);
  }

  return (
    <Stack
      boxShadow="lg"
      bg="brand.white"
      p={{ base: 2, sm: 6 }}
      borderRadius="md"
      mt={{ base: ".5rem", sm: "2rem" }}
      mb="2rem"
      spacing={4}
      as="form"
      onSubmit={handleSubmit(onSubmit)}
    >
      <Heading fontSize={{ base: "2xl", sm: "3xl" }}>
        ARTUR & HELDEN UMZÜGE UND TRANSPORTE
      </Heading>
      <Text
        color="brand.main"
        fontSize={{ base: "2xl", sm: "3xl" }}
        fontWeight="semibold"
      >
        Umzugsgutliste
      </Text>
      <Text
        fontSize={{ base: "xl", sm: "2xl" }}
        fontWeight="semibold"
        color="brand.darkBlue"
      >
        Ihr Umzugsunternehmen in Remscheid!
      </Text>
      <Text fontSize={{ base: "sm", md: "md" }}>
        Wir sind Ihr zuverlässiges Umzugsunternehmen in Remscheid und die
        Umgebung. Egal ob Möbelmontagen, insbesondere Küchen, Einlagerungen von
        Möbeln oder Kleintransporte – wir organisieren zuverlässig Ihren Umzug.
        Hier finden Sie unsere
      </Text>
      <Text fontSize={{ base: "sm", md: "md" }}>
        Umzugsgutliste. Bitte füllen Sie diese Liste gewissenhaft aus. Wir
        setzen uns anschließend mit Ihnen in Verbindung, um die Details wie
        Termin und Sonderwünsche zu besprechen. Bei weiteren Fragen helfen wir
        Ihnen gern weiter!
      </Text>
      <Alert
        status="warning"
        variant="left-accent"
        fontSize={{ base: "sm", md: "md" }}
        fontWeight="semibold"
      >
        <AlertIcon />
        Bitte füllen Sie die Umzugsliste gewissenhaft aus!
      </Alert>
      <Text fontWeight="semibold" fontSize="xl">
        Pflichtfelder sind mit einem Sternchen gekennzeichnet{" "}
        <Box color="red.400" fontSize="2xl" as="span">
          {"*"}
        </Box>
        .
      </Text>
      <Text fontSize={{ base: "sm", md: "md" }}>
        Falls Sie diese Liste nicht am Computer bearbeiten möchten, drucken Sie
        diese aus und schicken Sie die ausgefüllte Liste zu uns. Unser E-Mail
        Adresse ist info@arturundhelden.de
      </Text>

      {/* lOADING AND UNLOADING LOCATION DETAILS */}

      <SimpleGrid columns={{ base: 1, md: 2 }} spacing={6}>
        <Stack>
          <FormHeading heading="Beladeort" />

          <FormControl isRequired isInvalid={!!errors.loadingFrom}>
            <FormLabel>Name, Vorname:</FormLabel>
            <Input
              type="text"
              placeholder="Name, Vorname"
              {...register("loadingFrom")}
            />
            {errors.loadingFrom && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.loadingFrom.message}
              </FormErrorMessage>
            )}
          </FormControl>

          <FormControl isRequired isInvalid={!!errors.loadingFromZip}>
            <FormLabel>Straße, Hnr:</FormLabel>
            <Input
              type="text"
              placeholder="Straße, Hnr"
              {...register("loadingFromZip")}
            />
            {errors.loadingFromZip && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.loadingFromZip.message}
              </FormErrorMessage>
            )}
          </FormControl>
          <FormControl isRequired isInvalid={!!errors.loadingFromStreet}>
            <FormLabel>PLZ, Ort:*</FormLabel>
            <Input
              type="text"
              placeholder="PLZ, Ort"
              {...register("loadingFromStreet")}
            />
            {errors.loadingFromStreet && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.loadingFromStreet.message}
              </FormErrorMessage>
            )}
          </FormControl>

          <FormControl display="flex" gap={2}>
            <FormLabel>Enges Treppenhaus</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio value="1" {...register("loadingFromHasNarrowStaircase")}>
                  Ja
                </Radio>
                <Radio value="0" {...register("loadingFromHasNarrowStaircase")}>
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl display="flex" gap={2}>
            <FormLabel>Abtrageweg über 20m</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio value="1" {...register("loadingFromDistanceOver20m")}>
                  Ja
                </Radio>
                <Radio value="0" {...register("loadingFromDistanceOver20m")}>
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl display="flex" gap={2}>
            <FormLabel>Schrägaufyug</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio
                  value="1"
                  {...register("loadingFromHasInclinedElevator")}
                >
                  Ja
                </Radio>
                <Radio
                  value="0"
                  {...register("loadingFromHasInclinedElevator")}
                >
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl display="flex" gap={2}>
            <FormLabel>Fahrstuhl vorhanden</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio value="1" {...register("loadingFromHasElevator")}>
                  Ja
                </Radio>
                <Radio value="0" {...register("loadingFromHasElevator")}>
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl>
            <FormLabel>Errichtung einer Halteverbotszone:</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio
                  value="1"
                  {...register("loadingFromHasEstablishmentOfNoParkingZone")}
                >
                  Ja
                </Radio>
                <Radio
                  value="0"
                  {...register("loadingFromHasEstablishmentOfNoParkingZone")}
                >
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
        </Stack>
        <Stack>
          <FormHeading heading="Entladeort" />

          <FormControl isRequired isInvalid={!!errors.loadingTo}>
            <FormLabel>Name, Vorname:</FormLabel>
            <Input
              type="text"
              placeholder="Name, Vorname"
              {...register("loadingTo")}
            />
            {errors.loadingTo && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.loadingTo.message}
              </FormErrorMessage>
            )}
          </FormControl>

          <FormControl isRequired isInvalid={!!errors.loadingToZip}>
            <FormLabel>Straße, Hnr:</FormLabel>
            <Input
              type="text"
              placeholder="Straße, Hnr"
              {...register("loadingToZip")}
            />
            {errors.loadingToZip && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.loadingToZip.message}
              </FormErrorMessage>
            )}
          </FormControl>
          <FormControl isRequired isInvalid={!!errors.loadingToStreet}>
            <FormLabel>PLZ, Ort:*</FormLabel>
            <Input
              type="text"
              placeholder="PLZ, Ort"
              {...register("loadingToStreet")}
            />
            {errors.loadingToStreet && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.loadingToStreet.message}
              </FormErrorMessage>
            )}
          </FormControl>
          <FormControl display="flex" gap={2}>
            <FormLabel>Enges Treppenhaus</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio value="1" {...register("loadingToHasNarrowStaircase")}>
                  Ja
                </Radio>
                <Radio value="0" {...register("loadingToHasNarrowStaircase")}>
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl display="flex" gap={2}>
            <FormLabel>Abtrageweg über 20m</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio value="1" {...register("loadingToDistanceOver20m")}>
                  Ja
                </Radio>
                <Radio value="0" {...register("loadingToDistanceOver20m")}>
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl display="flex" gap={2}>
            <FormLabel>Schrägaufyug</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio value="1" {...register("loadingToHasInclinedElevator")}>
                  Ja
                </Radio>
                <Radio value="0" {...register("loadingToHasInclinedElevator")}>
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl display="flex" gap={2}>
            <FormLabel>Fahrstuhl vorhanden</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio value="1" {...register("loadingToHasElevator")}>
                  Ja
                </Radio>
                <Radio value="0" {...register("loadingToHasElevator")}>
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
          <CustomDivider />
          <FormControl>
            <FormLabel>Errichtung einer Halteverbotszone:</FormLabel>
            <RadioGroup defaultValue="0" colorScheme="main">
              <Stack direction="row">
                <Radio
                  value="1"
                  {...register("loadingToHasEstablishmentOfNoParkingZone")}
                >
                  Ja
                </Radio>
                <Radio
                  value="0"
                  {...register("loadingToHasEstablishmentOfNoParkingZone")}
                >
                  Nein
                </Radio>
              </Stack>
            </RadioGroup>
          </FormControl>
        </Stack>
      </SimpleGrid>

      <Stack>
        <FormHeading heading="Kontaktdaten (für evtl. Rückfrage)" />
        <Flex gap={6} flexDir={{ base: "column", md: "row" }}>
          <FormControl isRequired isInvalid={!!errors.contactTelephone}>
            <FormLabel>Telefon:</FormLabel>
            <Input
              type="tel"
              placeholder="Telefon"
              {...register("contactTelephone")}
            />
            {errors.contactTelephone && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.contactTelephone.message}
              </FormErrorMessage>
            )}
          </FormControl>
          <FormControl isRequired isInvalid={!!errors.contactEmail}>
            <FormLabel>E-Mail-Adresse:</FormLabel>
            <Input
              type="email"
              placeholder="E-Mail-Adresse"
              {...register("contactEmail")}
            />
            {errors.contactEmail && (
              <FormErrorMessage>
                <FormErrorIcon />
                {errors.contactEmail.message}
              </FormErrorMessage>
            )}
          </FormControl>
        </Flex>
        <FormControl isRequired isInvalid={!!errors.contactName}>
          <FormLabel>Durch Herrn/Frau:</FormLabel>
          <Input
            type="text"
            placeholder="Durch Herrn/Frau"
            {...register("contactName")}
          />
          {errors.contactName && (
            <FormErrorMessage>
              <FormErrorIcon />
              {errors.contactName.message}
            </FormErrorMessage>
          )}
        </FormControl>
      </Stack>

      <Stack>
        <FormHeading heading="Gewünschter Umzugstermin" />
        <FormControl isRequired isInvalid={!!errors.moveDate}>
          <FormLabel>TT.MM.JJJJ</FormLabel>
          <Input type="date" {...register("moveDate")} />
          {errors.moveDate && (
            <FormErrorMessage>
              <FormErrorIcon />
              {errors.moveDate.message}
            </FormErrorMessage>
          )}
        </FormControl>
      </Stack>

      <Text fontSize={{ base: "md", md: "xl" }} fontWeight="bold">
        Die in dieser Liste aufgeführten Raumeinheiten (RE) beziehen sich auf
        übliche Möbelgrößen und sind verbindliche Pauschalwerte.
      </Text>
      <Text fontSize={{ base: "md", md: "xl" }} fontWeight="bold">
        Andere Gegenstände, die nicht auf der Liste verzeichnet sind, sind{" "}
        <Box as="span" color="brand.red">
          im Freiraum
        </Box>{" "}
        unter dem jeweiligen Zimmer einzutragen{" "}
        <Box as="span" fontWeight="normal">
          (hierfür werden besondere RE mit Ihnen vereinbart).
        </Box>
      </Text>
      <Alert
        status="error"
        fontSize={{ base: "sm", md: "xl" }}
        fontWeight="bold"
        variant="left-accent"
      >
        <AlertIcon />
        Bitte beachten Sie, die angeführten RE dienen zur Orientierung. Sie
        erhalten von der Umzugsfirma im Kostenvoranschlag die nach Ihren Angaben
        errechneten RE!
      </Alert>
      <Text fontSize="lg" fontWeight="bold">
        1 RE{" "}
        <Box as="span" fontWeight="normal">
          entspricht
        </Box>{" "}
        0,1 m<Text as="sup">3</Text> <br />
        10 RE = 1 m<Text as="sup">3</Text>
      </Text>

      <Stack w="100%">
        <Accordion allowMultiple defaultIndex={[0]}>
          {tableData.map((section, i) => (
            <AccordionItem bg="brand.main" key={i}>
              <AccordionButton
                _expanded={{
                  bg: "brand.black",
                  color: "brand.white",
                }}
              >
                <Box flex="1" textAlign="left" fontSize="lg">
                  <FormHeading heading={section.heading} color="inherit" />
                </Box>
                <AccordionIcon boxSize={8} />
              </AccordionButton>

              <AccordionPanel pb={4} bg="brand.white">
                <Stack key={i} mt="1.2rem">
                  <TableContainer>
                    <Table
                      whiteSpace="wrap"
                      size={{ base: "xs", sm: "sm", md: "md" }}
                      variant="striped"
                      colorScheme="main"
                    >
                      <Thead color="brand.white">
                        <Tr bg="main.600" borderTopRadius="md">
                          <CustomTh
                            px=".5rem"
                            width={{ base: "20%", md: "30px" }}
                          >
                            Stück
                          </CustomTh>
                          <CustomTh width={{ base: "70%", md: "auto" }}>
                            Gegenstand
                          </CustomTh>
                          <CustomTh width={{ base: "10%", md: "auto" }}>
                            RE
                          </CustomTh>
                          <CustomTh px=".5rem">Ges.RE </CustomTh>
                        </Tr>
                      </Thead>

                      <Tbody>
                        {section.data.map((item, i) => (
                          <Tr key={i} alignItems="center" gap={2}>
                            <Td px=".5rem">
                              <Input
                                type="number"
                                boxShadow="none"
                                placeholder="0"
                                variant="flushed"
                                onChange={(e) =>
                                  modify(
                                    item.id,
                                    parseInt(e.target.value) || 0,
                                    section.type
                                  )
                                }
                              />
                            </Td>
                            <Td fontSize={{ base: "sm", sm: "lg" }}>
                              {item.gegenstand}
                            </Td>
                            <Td fontSize={{ base: "sm", sm: "lg" }}>
                              {item.RE}
                            </Td>
                            <Td
                              fontSize={{ base: "sm", sm: "lg" }}
                              fontWeight="semibold"
                              color="red.400"
                              px=".5rem"
                            >
                              {item.gesRE}
                            </Td>
                          </Tr>
                        ))}
                        <Tr>
                          <Td></Td>
                          <Td fontWeight="bold" textAlign="end">
                            Übertrag:
                          </Td>
                          <Td></Td>
                          <Td
                            fontSize={{ base: "md", md: "lg" }}
                            fontWeight="semibold"
                            color="red.400"
                            px=".5rem"
                          >
                            {getTotal(section.type)}
                          </Td>
                        </Tr>
                      </Tbody>
                      <Tfoot>
                        <Tr bg="brand.red">
                          <Th colSpan={2} color="brand.white">
                            Sonstiges
                          </Th>

                          <Th></Th>
                          <Th></Th>
                        </Tr>
                        <Tr>
                          <Td colSpan={4}>
                            <Stack>
                              <Text color="GrayText" fontSize="sm">
                                Alle anderen Details einfach schreiben,
                                Änderungen werden automatisch gespeichert!
                              </Text>
                              <SonstigenTextArea type={section.type} />
                            </Stack>
                          </Td>
                        </Tr>
                      </Tfoot>
                    </Table>
                  </TableContainer>
                </Stack>
              </AccordionPanel>
            </AccordionItem>
          ))}
        </Accordion>
      </Stack>

      <Stack mt="1.2rem">
        <FormHeading heading="Zusammenfassung" />
        <TableContainer>
          <Table
            whiteSpace="wrap"
            size={{ base: "xs", sm: "sm", md: "md" }}
            variant="striped"
            colorScheme="main"
            fontSize={{ base: "sm", sm: "md", md: "lg" }}
          >
            <Thead color="brand.white">
              <Tr py=".5rem" bg="main.600" borderTopRadius="md">
                <CustomTh px=".5rem">Raum</CustomTh>

                <CustomTh px=".5rem">Ges.RE </CustomTh>
              </Tr>
            </Thead>

            <Tbody>
              {tableData.map((item, i) => (
                <Tr key={i} alignItems="center" gap={2}>
                  <Td fontSize={{ base: "sm", sm: "lg" }}>{item.heading}</Td>

                  <Td
                    fontSize={{ base: "sm", sm: "lg" }}
                    fontWeight="semibold"
                    color="red.400"
                    px=".5rem"
                  >
                    {getSectionDataTotals(item.type)}
                  </Td>
                </Tr>
              ))}
              <Tr>
                <Td fontWeight="bold" textAlign="end">
                  Gesamtsumme:
                </Td>

                <Td
                  fontSize={{ base: "md", md: "lg" }}
                  fontWeight="semibold"
                  color="red.400"
                  px=".5rem"
                >
                  {getOverallTotal()}
                </Td>
              </Tr>
            </Tbody>
          </Table>
        </TableContainer>

        <Button
          colorScheme="main"
          size="lg"
          isLoading={isLoading}
          isDisabled={!isValid || isLoading}
          mt="1.2rem"
          type="submit"
        >
          Senden <SendIcon ml="1" />
        </Button>
      </Stack>
    </Stack>
  );
};

export default MoveForm;
